#ifndef INC_FUNCS_H
#define INC_FUNCS_H

/* Includes */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>

/* Defines */

#define INITIAL_VARIABLE_BUFFER_SIZE 8192
#define CHUNK_SIZE 4096

#define VARIABLE_BUFFER_INIT(x) { NULL,0,0,0,0 }
#define VARIABLE_BUFFER(x) struct varbuf x = VARIABLE_BUFFER_INIT(x)

#define BIT_TO_VALUE(bit) (1 << (bit))

static inline int set_bit(int bit,int *x){
	int value = BIT_TO_VALUE(bit);
	int old_bit = *x & value;
	*x|= value;
	return old_bit;
}

static inline int clear_bit(int bit,int *x){
	int value = BIT_TO_VALUE(bit);
	int old_bit = *x & value;
	*x&= ~value;
	return old_bit;
}

static inline int toggle_bit(int bit,int *x){
	int value = BIT_TO_VALUE(bit);
	int old_bit = *x & value;
	*x^= value;
	return old_bit;
}

static inline int test_bit(int bit,int *x){
	int value = BIT_TO_VALUE(bit);
	return *x & value;
}

static inline int make_bit(int bit,int *x,int on){
	if (on)
		return set_bit(bit,x);
	else
		return clear_bit(bit,x);
}

#define test_and_set_bit set_bit
#define test_and_clear_bit clear_bit
#define test_and_toggle_bit toggle_bit
#define test_and_make_bit make_bit

#define strnzcpy(d,s,n) do { strncpy(d,s,n); d[n] = '\0'; } while (0)
#define strnzcat(d,s,n) do { d[n + strlen(d)] = '\0'; strncat(d,s,n); } while (0)

/* Typedefs */

typedef void (* line_callback_t)(void *,char *);

/* Typedefs */

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned int dword;
typedef unsigned long long qword;
typedef unsigned long ip_t;

/* Early includes */

#include "list.h"

/* Functions */

/* Structs */

struct args {
	unsigned char *argbuf;
	unsigned char **arg; 
	unsigned char **argc;
	int count;
};

/* Variable buffer structure */
struct varbuf {
	char *buffer;			/* Actual buffer */
	size_t buffer_length;		/* Buffer data length */
	size_t allocated_length;	/* Actual allocated buffer size */
	size_t header_skip_length;	/* Length of starting buffer data skipped */
	size_t last_append_offset;	/* Start of last append */
};

/* Includes */

#ifndef CPROTO
#include "funcs.decl"
#endif

#endif
